/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-08-25
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <ifaddrs.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>

#include "Log.h"
#include "Address.h"

namespace GpCoreTools {

/*!
  \class Address qtbaddress.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

std::string Address::_interface;
Address Address::_me;
Address Address::_mask(0x00FFFFFF);
int Address::_maskSize=24;

/*!
  Description of constructor still missing
*/
Address::Address(int index, const Address& subnet)
{
  _value=(index << _maskSize) | (subnet & _mask)._value;
}

int Address::index() const
{
  return _value >> _maskSize;
}

void Address::setMaskSize(int m)
{
  _mask=(0xFFFFFFFF) >> (32-m);
  _maskSize=m;
}

/*!
  interface() must be available.
*/
void Address::identifyMe()
{
  _me._value=0;
  struct ifaddrs *ifaddr, *ifa;
  char host[NI_MAXHOST];
  if(getifaddrs(&ifaddr)==-1) {
    Log::write(2, "error getifaddrs: %s\n", strerror(errno));
  } else {
    // Walk through linked list, maintaining head pointer so we can free list later
    for(ifa=ifaddr; ifa!=NULL; ifa=ifa->ifa_next) {
      if(ifa->ifa_addr && ifa->ifa_addr->sa_family==AF_INET && strcmp(ifa->ifa_name, _interface.data())==0) {
        if(getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in), host, NI_MAXHOST, NULL, 0, NI_NUMERICHOST)!=0) {
          Log::write(2, "error getnameinfo: %s\n", strerror(errno));
          break;
        }
        _me.fromString(host);
        break;
      }
    }
    freeifaddrs(ifaddr);
  }
  if(_me.isValid()) {
    Log::write(0, "running on host %s\n", _me.toString().data());
  }
}

} // namespace GpCoreTools
