/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-07-01
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <string.h>
#include <stdio.h>

#include "ApplicationHelp.h"
#include "CoreApplication.h"

namespace GpCoreTools {

/*!
  \class ApplicationHelp ApplicationHelp.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
ApplicationHelp::ApplicationHelp()
{
}

/*!
  Description of destructor still missing
*/
ApplicationHelp::~ApplicationHelp()
{
}

void ApplicationHelp::addGroup(const char * title, const char * section)
{
  OptionGroup g;
  g.title=title;
  g.section=section;
  _options.push_back(g);
}

void ApplicationHelp::addOption(const char * option, const char * comments)
{
  OptionGroup& g=_options.back();
  Option o;
  o.option=option;
  o.comments=comments;
  g.options.push_back(o);
}

void ApplicationHelp::addExample(const char * command, const char * comments)
{
  Example o;
  o.command=command;
  o.comments=comments;
  _examples.push_back(o);
}

void ApplicationHelp::print(const OptionGroup& g)
{
  if(g.options.empty()) {
    print("\n"+g.title, "", 0);
  } else {
    print("\n"+g.title+" options:\n", "", 0);
    std::string indentStr;
    indentStr.assign(28, ' ');
    for(std::list<Option>::const_iterator it=g.options.begin(); it!=g.options.end(); it++ ) {
      const Option& o=*it;
      std::string cmt=o.comments;
      std::string cmtLine=getLine(cmt, CoreApplication::instance()->terminalCols()-28);
      if(o.option.length()<25) {
        printf("  %-25s %s\n", o.option.data(), cmtLine.data());
      } else {
        printf("  %s\n"
               "%s%s\n", o.option.data(), indentStr.data(), cmtLine.data());
      }
      print(cmt, "", 28);
    }
  }
}

/*!
  Returns the list of sections
*/
std::list<const char *> ApplicationHelp::sections()
{
  std::list<const char *> l;
  for(std::list<OptionGroup>::iterator it=_options.begin(); it!=_options.end(); it++ ) {
    l.push_back(it->section.data());
  }
  if( !_examples.empty()) {
    l.push_back("examples");
  }
  return l;
}

void ApplicationHelp::exec(const char * group)
{
  if(group) {
    if(strcmp(group,"html")==0) {
      execHtml();
      return;
    }
  }

  printf("\nUsage: %s %s\n\n", CoreApplication::instance()->applicationName(), _optionSummary.data());
  print(_comments, "", 0);
  if(group) {
    if(strcmp(group, "all")==0) {
      for(std::list<OptionGroup>::iterator it=_options.begin(); it!=_options.end(); it++ ) {
        print( *it);
      }
    } else if(strncmp(group, "no-", 3)==0) {
      group+=3;
      for(std::list<OptionGroup>::iterator it=_options.begin(); it!=_options.end(); it++ ) {
        if(strcmp(group, it->section.data())!=0) {
          print( *it);
        }
      }
    } else {
      for(std::list<OptionGroup>::iterator it=_options.begin(); it!=_options.end(); it++ ) {
        if(strcmp(group, it->section.data())==0) {
          print( *it);
        }
      }
    }
  } else if(!_options.empty()) {
    print(_options.front());
    if(_options.size()>1) print(_options.back());
  }
  if( !_examples.empty()) {
    printf("\nExamples:\n");
    for(std::list<Example>::iterator it=_examples.begin(); it!=_examples.end(); it++ ) {
      Example& o=*it;
      printf("\n");
      print(o.command, "", 9);
      printf("\n");
      print(o.comments, "", 2);
    }
  }
  printf("\nSee also:\n");
  if( !_seeAlso.empty()) {
    print(_seeAlso, "", 2);
  }
  print("More information at http://www.geopsy.org", "", 2);
  printf("\nAuthors:\n");
  print(CoreApplication::authors()+"\n", "", 2);
}

void replaceString(std::string& str, const char * find, const char * replace)
{
  size_t pos=0;
  int nFind=strlen(find);
  while(true) {
    pos=str.find(find, pos);
    if(pos==std::string::npos) return;
    str=str.replace(pos, nFind, replace);
    pos+=nFind;
  }
}

std::string htmlFilter(std::string str)
{
  replaceString(str, "&","&amp;");
  replaceString(str, "'","&apos;");
  replaceString(str, "\"","&quot;");
  replaceString(str, "<","&lt;");
  replaceString(str, ">","&gt;");
  replaceString(str, "\n  ","<li/>");
  replaceString(str, "\n","<br/>");
  return str;
}

void ApplicationHelp::execHtml()
{
  printf("<h2>Name</h2>\n");
  printf("%s - version %s\n", htmlFilter(CoreApplication::instance()->applicationName()).data(),
                              htmlFilter(CoreApplication::version(CoreApplication::instance()->applicationName())).data());
  printf("<h2>Synopsis</h2>\n");
  printf("%s %s\n", htmlFilter(CoreApplication::instance()->applicationName()).data(),
                    htmlFilter(_optionSummary).data());
  printf("<h2>Description</h2>\n");
  printf("<p>%s</p>\n", htmlFilter(_comments).data());
  for(std::list<OptionGroup>::iterator it=_options.begin(); it!=_options.end(); it++ ) {
    const OptionGroup& g=*it;
    if(g.options.empty()) {
      printf("<h3>%s</h3>\n", htmlFilter(g.title).data());
    } else {
      if(g.title.empty()) {
        printf("<h3>Options:</h3>\n");
      } else {
        printf("<h3>%s options:</h3>\n", htmlFilter(g.title).data());
      }
      printf("<dl compact>\n");
      for(std::list<Option>::const_iterator it=g.options.begin(); it!=g.options.end(); it++ ) {
        const Option& o=*it;
        printf("<dt><b>%s</b></dt>\n", htmlFilter(o.option).data());
        printf("<dd>%s</dd>\n", htmlFilter(o.comments).data());
      }
      printf("</dl>\n");
    }
  }
  if(!_examples.empty()) {
     printf("<h2>Examples</h2>\n");
     for(std::list<Example>::iterator it=_examples.begin(); it!=_examples.end(); it++ ) {
       Example& o=*it;
       printf("<pre>%s</pre>\n", htmlFilter(o.command).data());
       printf("<p>%s</p>\n", htmlFilter(o.comments).data());
     }
  }
  if(!_seeAlso.empty()) {
    printf("<h2>See Also</h2>\n");
    printf("<p>%s</p>\n",htmlFilter(_seeAlso).data());
  }
  printf("<h2>Authors</h2>\n");
  printf("%s", htmlFilter(CoreApplication::authors()).data());
}

/*!
  Print string \a p to stdout. Prefix \a linePrefix and indent \a indent are
  inserted before each line. app.terminalCols() is the maximum number of
  characters per line.
*/
void ApplicationHelp::print(std::string p, const std::string& linePrefix, int indent)
{
  std::string line;
  std::string mainIndentStr, customIndentStr;
  bool newLine=true;
  mainIndentStr.assign(indent, ' ');
  while(!p.empty()) {
    bool lastNewLine=newLine;
    line=getLine(p, CoreApplication::instance()->terminalCols()-indent-linePrefix.size()-customIndentStr.size(), &newLine);
    printf("%s%s%s%s\n", linePrefix.data(), mainIndentStr.data(), customIndentStr.data(), line.data());
    if(lastNewLine && !newLine) {
      // Set indent by the position of the last double space
      customIndentStr=getCustomIndent(line);
      newLine=false;
    } else if(newLine) {
      customIndentStr="";
    }
  }
}

std::string ApplicationHelp::getCustomIndent(std::string& text)
{
  int i=text.rfind("  ");
  if(i>-1) {
    std::string customIndentStr;
    customIndentStr.assign(i+2, ' ');
    return customIndentStr;
  } else {
    return "";
  }
  /*int nBlanks=0;
  int n=text.count();
  while(nBlanks<n && text[nBlanks]==' ') nBlanks++;
  if(nBlanks>0) {
    std::string customIndentStr;
    customIndentStr.fill(' ',nBlanks);
    text=text.mid(nBlanks);
    return customIndentStr;
  } else {
    return std::string::null;
  }*/
}

/*!
  Returns the first line of \a text. The first line is removed from \a text.
  \a newLine is set to true if a '\n' is encountered else it is set to false.
*/
std::string ApplicationHelp::getLine(std::string& text, int maxLength, bool * newLine)
{
  if(maxLength<10) {
    std::string line=text;
    text="";
    return line;
  }
  std::string line=text;
  int index=text.find_first_of("\n");
  if(index>-1) {
    if(index<=maxLength) {
      text=text.substr(index+1);
      line=line.substr(0, index);
      if(newLine) *newLine=true;
      return line;
    }
  }
  if((int)text.length()<=maxLength) {
    text.clear();
    if(newLine) *newLine=false;
    return line;
  }
  line=line.substr(0, maxLength);
  index=line.find_last_of(" ");
  if(maxLength>20 && index>maxLength-20) {
    text=text.substr(index+1); // skip the blank
    line=line.substr(0, index);
    if(newLine) *newLine=false;
    return line;
  }
  text=text.substr(0, maxLength);
  if(newLine) *newLine=false;
  return line;
}

} // namespace GpCoreTools
