/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-05-30
**  Authors:
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#ifndef BYTEORDER_H
#define BYTEORDER_H

#include "GpCoreToolsDLLExport.h"

//
// Can be BIG_ENDIAN or LITTLE_ENDIAN, set with -D option in gcc command line
//
#ifndef BYTE_ORDER
#  define BYTE_ORDER LITTLE_ENDIAN
#endif

namespace GpCoreTools {

  class GPCORETOOLS_EXPORT ByteOrder
  {
  public:
    static inline int bigEndianToNative(int v);
    static inline int littleEndianToNative(int v);

    static inline short bigEndianToNative(const unsigned short& v);
    static inline short littleEndianToNative(const unsigned short& v);

    static int swap(int v);
    static short swap(const unsigned short& v);
    static inline void swap(char& v1, char& v2);
    static inline void swap(unsigned char& v1, unsigned char& v2);
    static inline void swap(char * v);
    static inline void swap(unsigned char * v);
  };

  inline int ByteOrder::bigEndianToNative(int v)
  {
#if BYTE_ORDER == LITTLE_ENDIAN
    return swap(v);
#else
    return v;
#endif
  }

  inline int ByteOrder::littleEndianToNative(int v)
  {
#if BYTE_ORDER == BIG_ENDIAN
    return swap(v);
#else
    return v;
#endif
  }

  inline short ByteOrder::bigEndianToNative(const unsigned short& v)
  {
#if BYTE_ORDER == LITTLE_ENDIAN
    return swap(v);
#else
    return v;
#endif
  }

  inline short ByteOrder::littleEndianToNative(const unsigned short& v)
  {
#if BYTE_ORDER == BIG_ENDIAN
    return swap(v);
#else
    return v;
#endif
  }

  inline void ByteOrder::swap(char& v1, char& v2)
  {
    char tmp=v1;
    v1=v2;
    v2=tmp;
  }

  inline void ByteOrder::swap(unsigned char& v1, unsigned char& v2)
  {
    char tmp=v1;
    v1=v2;
    v2=tmp;
  }

  inline void ByteOrder::swap(char * v)
  {
    swap(v[0], v[3]);
    swap(v[1], v[2]);
  }

  inline void ByteOrder::swap(unsigned char * v)
  {
    swap(v[0], v[3]);
    swap(v[1], v[2]);
  }

} // namespace GpCoreTools

#endif // BYTEORDER_H
