/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-06-29
**  Authors:
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#ifndef DAEMONAPPLICATION_H
#define DAEMONAPPLICATION_H

#include "CoreApplication.h"
#include "Leds.h"
#include "Log.h"
#include "GpCoreToolsDLLExport.h"

namespace GpCoreTools {

  class GPCORETOOLS_EXPORT DaemonApplication : public CoreApplication
  {
  public:
    DaemonApplication(int & argc, char ** argv, ApplicationHelp * ( *help) (), bool reportBugs=true);
    ~DaemonApplication();

    virtual void close();

    void fork();
    void start();
    const std::string& basePath() const {return _basePath;}
    static DaemonApplication * instance() {return _self;}

    void setFork(bool df) {_doFork=df;}
  protected:
    virtual void parentProcessWait() {}
    virtual void installInterruptSignals();
    static void childInterrupted(int signum);
    static void parentInterrupted(int signum);
  private:
    void checkPidFile();
    void writePidFile(pid_t pid);
    void checkBasePath();
    void initLogFile();
    void showHelp(int& i, int argc, char ** argv, ApplicationHelp * ( *help) ());
    void setHelp(ApplicationHelp * h);

    static std::string pidFile(const char * applicationName);

    bool _doFork;
    std::string _basePath, _logFileName;
    Log * _log;
    Leds * _leds;
    static DaemonApplication * _self;
  };

} // namespace GpCoreTools

#endif // DAEMONAPPLICATION_H
