/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-06-28
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <string.h>
#include <assert.h>
#include <errno.h>

#include "DynamicBuffer.h"
#include "Log.h"
#include "Trace.h"

namespace GpCoreTools {

/*!
  \class DynamicBuffer qtbdynamicbuffer.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
DynamicBuffer::DynamicBuffer(int fileDescriptor)
{
  TRACE;
  _fileDescriptor=fileDescriptor;
  _bufferCapacity=64;
  _buffer=(char *)malloc(_bufferCapacity);
  _bufferIn=0;
  _bufferOut=0;
  _debugIgnoreByteCount=0;
  _debugIgnoreByteColumnCount=8;
  _debugIgnoreByteBuffer=0;
  _debugIgnoreBytePointer=0;
}

/*!
  Description of destructor still missing
*/
DynamicBuffer::~DynamicBuffer()
{
  TRACE;
  delete [] _buffer;
  delete [] _debugIgnoreByteBuffer;
}

void DynamicBuffer::close()
{
  TRACE;
  if(_fileDescriptor>0) {
    ::close(_fileDescriptor);
  }
  _fileDescriptor=-1;
}

/*!
  Returns false if data cannot be read.
*/
bool DynamicBuffer::read()
{
  TRACE;
  // Makes as much space as possible without moving anything
  if(_bufferOut==_bufferIn) { // Everything is processed, force return to 0
    _bufferIn=0;
    _bufferOut=0;
  }
  // Reads as much as possible
  int maxNRead=0, nRead=0;
  while(nRead==maxNRead) {
    if(_bufferIn==_bufferCapacity) {
      _bufferCapacity=_bufferCapacity << 1;
      Log::write(10, "increase capacity to %i\n", _bufferCapacity);
      _buffer=(char *)realloc(_buffer, _bufferCapacity);
    }
    maxNRead=_bufferCapacity-_bufferIn;
    nRead=::read(_fileDescriptor, _buffer+_bufferIn, maxNRead);
    if(nRead>0) {
      _bufferIn+=nRead;
    } else {
      break;
    }
  }
  // Processes read bytes
  //Log::write(10, "buffer filled up to %i (%.1f %%)\n", _bufferIn, (float)_bufferIn*100.0/_bufferCapacity);
  int nProcessed=0;
  do {
    nProcessed=bytesAvailable(_buffer+_bufferOut, _bufferIn-_bufferOut);
    _bufferOut+=nProcessed;
  } while (nProcessed>0 && _bufferOut<_bufferIn);
  assert(_bufferOut<=_bufferIn);
  return true;
}

void DynamicBuffer::debugBuffer() const
{
  TRACE;
  Log::write(0, "DynamicBuffer::debugBuffer\n");
  Log::write(0, "  out %i in %i (available %i) capacity %i\n",
             _bufferOut, _bufferIn, _bufferIn-_bufferOut, _bufferCapacity);
  char * sbuf=new char[_bufferIn*4+8]; // 3 char/byte, 5+1 char for address and \n every 8 bytes (counted as 1 char/byte)
                                       // in case of a line with 1 byte: 5 (address)+3 (1 bytes)+1 (\n)=9<1*4+8
  char * psbuf=sbuf;
  int nLine=7;
  for(int i=0; i<_bufferIn; i++) {
    if(nLine==7) {
      nLine=0;
      sprintf(psbuf, "\n%05i", i);
      psbuf+=6;
    } else {
      nLine++;
    }
    sprintf(sbuf, " %02hhX", _buffer[i]);
    psbuf+=3;
  }
  sprintf(psbuf, "\n");
  Log::write(0, sbuf);
  delete [] sbuf;
}

void DynamicBuffer::debugIgnoreByte()
{
  TRACE;
  if(_debugIgnoreByteCount==0) {
    Log::write(0, "start ignoring bytes");
    _debugIgnoreByteBuffer=new char[256*4+8];
    _debugIgnoreBytePointer=_debugIgnoreByteBuffer;
  }
  if(_debugIgnoreByteColumnCount==8) {
    _debugIgnoreByteColumnCount=1;
    sprintf(_debugIgnoreBytePointer, "\n%05i", _debugIgnoreByteCount);
    _debugIgnoreBytePointer+=6;
  } else {
    _debugIgnoreByteColumnCount++;
  }
  sprintf(_debugIgnoreBytePointer, " %02hhX", _buffer[_bufferOut]);
  _debugIgnoreBytePointer+=3;
  _debugIgnoreByteCount++;
  if(_debugIgnoreByteCount==256) {
    sprintf(_debugIgnoreBytePointer, "\n");
    Log::write(0, _debugIgnoreByteBuffer);
    _debugIgnoreBytePointer=_debugIgnoreByteBuffer;
  }
}

void DynamicBuffer::debugEndIgnoreByte()
{
  if(_debugIgnoreByteCount>0) {
    _debugIgnoreByteColumnCount=8;
    _debugIgnoreByteCount=0;
    sprintf(_debugIgnoreBytePointer, "\n");
    Log::write(0, _debugIgnoreByteBuffer);
    delete [] _debugIgnoreByteBuffer;
    _debugIgnoreByteBuffer=0;
  }
}

} // namespace GpCoreTools
