/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-06-26
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#ifndef FLETCHERCHECKSUM_H
#define FLETCHERCHECKSUM_H

#include <stdio.h>

#include "GpCoreToolsDLLExport.h"

namespace GpCoreTools {

  class GPCORETOOLS_EXPORT FletcherChecksum
  {
  public:
    FletcherChecksum() {_values.a=0; _values.b=0;}
    FletcherChecksum(unsigned char a, unsigned char b) {_values.a=a; _values.b=b;}
    FletcherChecksum(const FletcherChecksum& o) {_raw=o._raw;}

    bool operator==(const FletcherChecksum& o) const {return _raw==o._raw;}

    void reset() {_values.a=0; _values.b=0;}
    inline void add(const char * buffer, int length);

    unsigned char a() const {return _values.a;}
    unsigned char b() const {return _values.b;}
  private:
    union {
      struct {
        unsigned char a, b;
      } _values;
      unsigned char _bytes[2];
      unsigned short _raw;
    };
  };

  inline void FletcherChecksum::add(const char * buffer, int length)
  {
    for(int i=0; i<length; i++) {
      _values.a+=buffer[i];
      _values.b+=_values.a;
    }
  }

} // namespace GpCoreTools

#endif // FLETCHERCHECKSUM_H
