/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-08-24
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>
#include <unistd.h>
#include <string.h>
#include <string>
#include <errno.h>

#include "Leds.h"
#include "Log.h"

namespace GpCoreTools {

/*!
  \class Leds qtbleds.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

Leds * Leds::_self=0;

static const char * ledPath="/sys/class/leds/alix:";
static const char * ledIndex[]={"1/", "2/", "3/"};

bool Leds::LedTimer::exec()
{
  Leds::power(_index, _on);
  return false;
}

/*!
  Description of constructor still missing
*/
Leds::Leds()
{
  assert(!_self);

  // Test all 3 lights
  if(!test(0) || !test(1) || !test(2)) {
    Log::write(1, "no led available\n");
    // Do not initialize _self, leds are not available
    return;
  }

  _self=this;
  _timer[0]._index=0;
  _timer[1]._index=1;
  _timer[2]._index=2;
}

bool Leds::test(int index)
{
  std::string p=ledPath;
  p+=ledIndex[index];
  p+="brightness";
  int fd=open(p.data(), O_RDONLY);
  if(fd>0) {
    char state;
    if(read(fd, &state, 1)==1) {
      close(fd);
      return true;
    }
    close(fd);
  }
  return false;
}

void Leds::power(int index, bool on)
{
  if(_self) {
    _self->setTrigger(index, "default-on");
    _self->powerInternal(index, on);
  }
}

void Leds::flash(int index, int msec, bool on)
{
  if(_self) {
    LedTimer& t=_self->_timer[index];
    t._on=!on;
    t.setInterval(msec);
    t.start();
    _self->setTrigger(index, "default-on");
    power(index, on);
  }
}

void Leds::timer(int index, int msecOn, int msecOff)
{
  if(_self) {
    _self->setTrigger(index, "timer");
    _self->powerInternal(index, true);
    _self->setDelayOn(index, msecOn);
    _self->setDelayOff(index, msecOff);
  }
}

void Leds::setTrigger(int index, const char * type)
{
  std::string p=ledPath;
  p+=ledIndex[index];
  p+="trigger";
  int fd=open(p.data(), O_WRONLY);
  if(fd>0) {
    int n=strlen(type);
    if(write(fd, type, n)!=n) {
      Log::write(1, "error setting led trigger - %s\n", strerror(errno));
    }
    close(fd);
  }
}

void Leds::powerInternal(int index, bool on)
{
  std::string p=ledPath;
  p+=ledIndex[index];
  p+="brightness";
  int fd=open(p.data(), O_WRONLY);
  if(fd>0) {
    if(write(fd, on ? "1" : "0", 1)!=1) {
      Log::write(1, "error setting led power - %s\n", strerror(errno));
    }
    close(fd);
  }
}

void Leds::setDelayOn(int index, int msec)
{
  std::string p=ledPath;
  p+=ledIndex[index];
  p+="delay_on";
  int fd=open(p.data(), O_WRONLY);
  if(fd>0) {
    char buf[16];
    sprintf(buf, "%i", msec);
    int n=strlen(buf);
    if(write(fd, buf, n)!=n) {
      Log::write(1, "error setting led delay on - %s\n", strerror(errno));
    }
    close(fd);
  }
}

void Leds::setDelayOff(int index, int msec)
{
  std::string p=ledPath;
  p+=ledIndex[index];
  p+="delay_off";
  int fd=open(p.data(), O_WRONLY);
  if(fd>0) {
    char buf[16];
    sprintf(buf, "%i", msec);
    int n=strlen(buf);
    if(write(fd, buf, n)!=n) {
      Log::write(1, "error setting led delay off - %s\n", strerror(errno));
    }
    close(fd);
  }
}

} // namespace GpCoreTools
