/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-04-21
**  Authors:
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#ifndef LINEARREGRESSION_H
#define LINEARREGRESSION_H

#include <vector>

#include "GpCoreToolsDLLExport.h"

namespace GpCoreTools {

  class GPCORETOOLS_EXPORT LinearRegression
  {
  public:
    LinearRegression() {_values=0; reset();}
    ~LinearRegression() {delete _values;}

    void add(double x, double y);
    void reset(int n=0);

    bool calculate();
    float stddev() const;
    int badValue(double deviation) const;
    void remove(int i);
    void removeFirst();

    unsigned int count() const {return _values->size();}
    double a() const {return _a;}
    double b() const {return _b;}
    double at(double x) const {return _a*(x-_x0)+_b;}
  private:
    double _sumX, _sumY, _sumXY, _sumX2, _x0;
    double _a, _b;
    std::vector<std::pair<double, double> > * _values;
  };

} // namespace GpCoreTools

#endif // LINEARREGRESSION_H
