/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-07-12
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <assert.h>
#include <fcntl.h>
#ifdef SYSLOG
#include <syslog.h>
#endif

#include "CoreApplication.h"
#include "Log.h"

namespace GpCoreTools {

/*!
  \class Log Log.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

FILE * Log::_stream=stderr;
int Log::_verbosity=0;
pthread_mutex_t Log::_mutex;

/*!
  Init log to \a stream. If null, the default stderr is not changed.
*/
Log::Log(FILE * stream)
{
#ifdef SYSLOG
  openlog (CoreApplication::instance()->applicationName(), LOG_NDELAY | LOG_PID, LOG_DAEMON);
#else
  if(stream) {
    _stream=stream;
  }
#endif
}

/*!
  Description of destructor still missing
*/
Log::~Log()
{
#ifdef SYSLOG
  closelog();
#else
  if(_stream!=stderr && _stream!=stdin) {
    fclose(_stream);
  }
#endif
}

/*!
  Description of constructor still missing
*/
void Log::setFile(const char * fileName)
{
#ifndef SYSLOG
  if(!fileName) {
    return;
  }
  if(_stream!=stderr && _stream!=stdin) {
    fclose(_stream);
  }

  int fd=open(fileName, O_WRONLY | O_NONBLOCK | O_CREAT | O_APPEND, S_IRWXU | S_IRGRP | S_IROTH);
  if(fd>=0) {
    _stream=fdopen(fd, "a");
    assert(_stream);
  } else {
    _stream=stderr;
    write(0, "failed to open log file %s\n",fileName);
  }
#endif
}

void Log::timeStamp(char * buffer32)
{
  time_t t=time(0);
  strftime(buffer32, 32, "%a %b %d %H:%M:%S %Y: ", gmtime(&t));
}

void Log::write(int level, const char * cformat,...)
{
  if(level<=_verbosity) {
    va_list argptr;
    va_start(argptr, cformat);
#ifdef SYSLOG
    vsyslog(LOG_INFO, cformat, argptr);
#else
    // Print time stamp
    char timeBuf[32];
    timeStamp(timeBuf);
    pthread_mutex_lock(&_mutex);
    // Print message
    unsigned int n=strlen(timeBuf);
    if(fwrite(timeBuf, 1, n, _stream)==n) {
      vfprintf(_stream, cformat, argptr);
      fflush(_stream);
    }
    pthread_mutex_unlock(&_mutex);
#endif
    va_end(argptr);
  }
}

/*!
  Used only when no mutex can be used, for instance during signaled exceptions.
*/
void Log::writeNoMutex(int level, const char * cformat,...)
{
  if(level<=_verbosity) {
    va_list argptr;
    va_start(argptr, cformat);
#ifdef SYSLOG
    vsyslog(LOG_INFO, cformat, argptr);
#else
    // Print time stamp
    char timeBuf[32];
    timeStamp(timeBuf);
    unsigned int n=strlen(timeBuf);
    // Print message
    if(fwrite(timeBuf, 1, n, _stream)==n) {
      vfprintf(_stream, cformat, argptr);
      fflush(_stream);
    }
#endif
    va_end(argptr);
  }
}

void Log::writeNoStamp(int level, const char * cformat,...)
{
  if(level<=_verbosity) {
    va_list argptr;
    va_start(argptr, cformat);
#ifdef SYSLOG
    vsyslog(LOG_INFO, cformat, argptr);
#else
    // Print message
    pthread_mutex_lock(&_mutex);
    vfprintf(_stream, cformat, argptr);
    fflush(_stream);
    pthread_mutex_unlock(&_mutex);
#endif
    va_end(argptr);
  }
}

} // namespace GpCoreTools
