/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-07-12
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#ifndef LOG_H
#define LOG_H

#include <stdio.h>
#include <stdarg.h>
#include <string>

#include "GpCoreToolsDLLExport.h"

namespace GpCoreTools {

  class GPCORETOOLS_EXPORT Log
  {
  public:
    Log(FILE * stream=0);
    ~Log();

    void setFile(const char * fileName);

    static void setVerbosity(int v) {_verbosity=v;}
    static int verbosity() {return _verbosity;}

    static FILE * stream() {return _stream;}
    static void timeStamp(char * buffer32);
    static void write(int level, const char * cformat,...);
    static void writeNoMutex(int level, const char * cformat,...);
    static void writeNoStamp(int level, const char * cformat,...);
  private:
    static FILE * _stream;
    static int _verbosity;
    static pthread_mutex_t _mutex;
  };

} // namespace GpCoreTools

#endif // LOG_H
