/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-06-29
**  Authors:
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#ifndef MESSAGECLASSID_H
#define MESSAGECLASSID_H

#include "MessageClassIdHeader.h"

namespace GpCoreTools {

#pragma pack(1)     /* set alignment to 1 byte boundary */

  template <class Payload> class GPCORETOOLS_EXPORT MessageClassId
  {
  public:
    inline MessageClassId(unsigned char msgClass, unsigned char msgId)
      : _head(msgClass, msgId, sizeof(Payload)), _payload(), _tail() {}
    inline MessageClassId(unsigned char msgClass, unsigned char msgId, const Payload& p)
      : _head(msgClass, msgId, sizeof(Payload)), _payload(p), _tail() {}

    inline bool isLittleEndianValid() const;
    inline bool isBigEndianValid() const;

    inline void littleEndianValues();
    inline void bigEndianValues();

    const MessageClassIdHeader& header() const {return _head;}
    Payload& payload() {return _payload;}
    const Payload& payload() const {return _payload;}
    const FletcherChecksum& checksum() const {return _tail;}

    const char * buffer() const {return reinterpret_cast<const char *>(this);}
    const char * payloadBuffer() const {return reinterpret_cast<const char *>(&_payload);}
  private:
    MessageClassIdHeader _head;
    Payload _payload;
    FletcherChecksum _tail;
  };

  template <class Payload>
  inline void MessageClassId<Payload>::littleEndianValues()
  {
    _head.littleEndianValues();
    _payload.littleEndianValues();
    _tail.add(reinterpret_cast<const char *>(_head.header()), MessageClassIdHeader::HeaderSize);
    _tail.add(reinterpret_cast<const char *>(&_payload), sizeof(Payload));
  }

  template <class Payload>
  inline void MessageClassId<Payload>::bigEndianValues()
  {
    _head.bigEndianValues();
    _payload.bigEndianValues();
    _tail.add(reinterpret_cast<const char *>(_head.header()), MessageClassIdHeader::HeaderSize);
    _tail.add(reinterpret_cast<const char *>(&_payload), sizeof(Payload));
  }

  template <class Payload>
  inline bool MessageClassId<Payload>::isLittleEndianValid() const
  {
    return _head.isValid() &&
           _head.lengthLittleEndian()==sizeof(Payload) &&
           _head.isValid(_tail, payloadBuffer(), sizeof(Payload));
  }

  template <class Payload>
  inline bool MessageClassId<Payload>::isBigEndianValid() const
  {
    return _head.isValid() &&
           _head.lengthBigEndian()==sizeof(Payload) &&
           _head.isValid(_tail, payloadBuffer(), sizeof(Payload));
  }

#pragma pack()   // restore original alignment

} // namespace GpCoreTools

#endif // MESSAGECLASSID_H
