/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-06-29
**  Authors:
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#ifndef MESSAGECLASSIDONLY_H
#define MESSAGECLASSIDONLY_H

#include "MessageClassIdHeader.h"

namespace GpCoreTools {

#pragma pack(1)     /* set alignment to 1 byte boundary */

class GPCORETOOLS_EXPORT MessageClassIdOnly
  {
  public:
    inline MessageClassIdOnly(unsigned char msgClass, unsigned char msgId);

    const MessageClassIdHeader& header() const {return _head;}
    const FletcherChecksum& checksum() const {return _tail;}

    const char * buffer() const {return reinterpret_cast<const char *>(this);}
  private:
    MessageClassIdHeader _head;
    FletcherChecksum _tail;
  };

  inline MessageClassIdOnly::MessageClassIdOnly(unsigned char msgClass, unsigned char msgId)
    : _head(msgClass, msgId, 0), _tail()
  {
    _tail.add(reinterpret_cast<const char *>(_head.header()), MessageClassIdHeader::HeaderSize);
  }

#pragma pack()   // restore original alignment

} // namespace GpCoreTools

#endif // MESSAGECLASSIDONLY_H
