/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-06-20
**  Authors:
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#ifndef MESSAGERAWHEADER_H
#define MESSAGERAWHEADER_H

#include "GpCoreToolsDLLExport.h"
#include "FletcherChecksum.h"
#include "ByteOrder.h"

namespace GpCoreTools {

#pragma pack(1)     /* set alignment to 1 byte boundary */

  class GPCORETOOLS_EXPORT MessageRawHeader
  {
  public:
    MessageRawHeader() {}
    inline MessageRawHeader(unsigned int length);
    inline MessageRawHeader(const MessageRawHeader& o);

    enum {SynchronizationSize=2, HeaderSize=2, CheckSumSize=2};

    bool isValid() const {return _sync[0]==0xB5 && _sync[1]==0x62;}
    inline bool isValid(const FletcherChecksum& cksum, const char * payload=0,
                        unsigned short payloadLength=0) const;

    inline void littleEndianValues();
    inline void bigEndianValues();

    const char * header() const {return _bytes;}

    unsigned int lengthLittleEndian() const {return ByteOrder::littleEndianToNative(_values.length);}
    unsigned int lengthBigEndian() const {return ByteOrder::bigEndianToNative(_values.length);}
  protected:
    unsigned char _sync[2];
    union {
      struct {
        unsigned short length;
      } _values;
      char _bytes[HeaderSize];
    };
  };

  inline MessageRawHeader::MessageRawHeader(unsigned int length)
  {
    _sync[0]=0xB5;
    _sync[1]=0x62;
    _values.length=length;
  }

  inline MessageRawHeader::MessageRawHeader(const MessageRawHeader& o)
  {
    _sync[0]=o._sync[0];
    _sync[1]=o._sync[1];
    _values.length=o._values.length;
  }

  inline void MessageRawHeader::bigEndianValues()
  {
#if BYTE_ORDER==LITTLE_ENDIAN
    ByteOrder::swap(_bytes[0], _bytes[1]);
#endif
  }

  inline void MessageRawHeader::littleEndianValues()
  {
#if BYTE_ORDER==BIG_ENDIAN
    ByteOrder::swap(_bytes[0], _bytes[1]);
#endif
  }

  inline bool MessageRawHeader::isValid(const FletcherChecksum& cksum, const char * payload,
                                        unsigned short payloadLength) const
  {
    FletcherChecksum mycs;
    mycs.add(reinterpret_cast<const char *>(header()), HeaderSize);
    if(payload) {
      mycs.add(payload, payloadLength);
    }
    return cksum==mycs;
  }

#pragma pack()   // restore original alignment

} // namespace GpCoreTools

#endif // MESSAGERAWHEADER_H
