/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-06-21
**  Authors:
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#include "Trace.h"
#include "MovingStatistics.h"

namespace GpCoreTools {

  /*!
    \class MovingStatistics MovingStatistics.h
    \brief Brief description of class still missing

    Full description of class still missing
  */

  /*!
    Description of constructor still missing
  */
  MovingStatistics::MovingStatistics()
    : Statistics()
  {
    TRACE;
    _values=0;
    _currentIndex=0;
    _size=0;
  }

  /*!
    Description of constructor still missing
  */
  MovingStatistics::MovingStatistics(int size)
    : Statistics()
  {
    TRACE;
    _values=new double[size];
    _currentIndex=0;
    _size=size;
  }

  /*!
    Description of destructor still missing
  */
  MovingStatistics::~MovingStatistics()
  {
    TRACE;
    delete [] _values;
  }

  /*!
    Add a new value. Minimum and maximum are calculated.
  */
  void MovingStatistics::addMinMax(double val)
  {
    if(_count<_size) {
      Statistics::add(val);
      _values[_currentIndex]=val;
    } else {
      if(_currentIndex==_size) {
        _currentIndex=0;
      }
      double oldVal=_values[_currentIndex];
      _sum+=val-oldVal;
      _sum2+=val*val-oldVal*oldVal;
      _values[_currentIndex]=val;
      if(oldVal==_minimum || oldVal==_maximum) {
        for(int i=0; i<_size; i++) {
          val=_values[i];
          if(val<_minimum) _minimum=val;
          if(val>_maximum) _maximum=val;
        }
      } else {
        if(val<_minimum) _minimum=val;
        if(val>_maximum) _maximum=val;
      }
    }
    _currentIndex++;
  }

  /*!
    Add a new value. Minimum and maximum are not calculated.
  */
  void MovingStatistics::add(double val)
  {
    if(_count<_size) {
      _sum+=val;
      _sum2+=val*val;
      _count++;
    } else {
      if(_currentIndex==_size) {
        _currentIndex=0;
      }
      double oldVal=_values[_currentIndex];
      _sum+=val-oldVal;
      _sum2+=val*val-oldVal*oldVal;
    }
    _values[_currentIndex]=val;
    _currentIndex++;
  }


  void MovingStatistics::set(double val)
  {
    Statistics::set(val);
    _currentIndex=1;
  }

  void MovingStatistics::reset(int size)
  {
    if(size>_size) {
      delete [] _values;
      if(size>0) {
        _values=new double[size];
      } else {
        _values=0;
      }
    }
    _size=size;
    Statistics::reset();
    _currentIndex=0;
  }

} // namespace GpCoreTools
