/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-06-27
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <assert.h>

#include "PThread.h"
#include "Log.h"

namespace GpCoreTools {

/*!
  \class PThread PThread.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

pthread_mutex_t PThread::_threadsMutex;
std::map<pthread_t, PThread *> PThread::_threads;

void PThread::init()
{
  assert(pthread_mutex_init(&_threadsMutex, 0)==0);
}

PThread::PThread()
{
  _eventLoop=0;
}

/*!
  Usually never called. EventLoop's constructor does it for you.
  You can create only one event loop per thread.
*/
void PThread::setEventLoop(EventLoop * l)
{
  assert(!_eventLoop);
  _eventLoop=l;
}

void PThread::start()
{
  pthread_attr_t attr;
  pthread_attr_init(&attr);
  pthread_create(&_id, &attr, run, this);
}

void PThread::stop()
{
  if(_eventLoop) {
    _eventLoop->terminate();
  }
}

void PThread::wait()
{
  pthread_join(_id, 0);
}

void * PThread::run(void * arg)
{
  PThread * t=static_cast<PThread *>(arg);
  Log::write(5, "enter thread id %u\n", t->id());

  // Update list of running threads
  pthread_mutex_lock(&_threadsMutex);
  _threads.insert(std::map<pthread_t, PThread *>::value_type(t->id(), t));
  pthread_mutex_unlock(&_threadsMutex);

  t->run();

  // Update list of running threads
  pthread_mutex_lock(&_threadsMutex);
  _threads.erase(t->id());
  pthread_mutex_unlock(&_threadsMutex);

  Log::write(5, "exit from thread id %u\n", t->id());
  return 0;
}

PThread * PThread::currentThread()
{
  pthread_mutex_lock(&_threadsMutex);
  pthread_t id=pthread_self();
  std::map<pthread_t, PThread *>::iterator it;
  it=_threads.find(id);
  PThread * t;
  if(it!=_threads.end()) {
    t=it->second;
  } else {
    t=0;
  }
  pthread_mutex_unlock(&_threadsMutex);
  return t;
}

#ifdef TRACE_ENABLED
std::list< std::vector<const TraceStamp *> * > PThread::stacks()
{
  std::list< std::vector<const TraceStamp *> * > list;
  list.push_back(Trace::mainStack());
  for(std::map<pthread_t, PThread *>::iterator it=_threads.begin(); it!=_threads.end(); it++ ) {
    list.push_back(it->second->stack());
  }
  return list;
}
#endif

std::list< std::vector<TraceBug *> * > PThread::bugStacks()
{
  std::list< std::vector<TraceBug *> * > list;
  list.push_back(TraceBug::mainStack());
  for(std::map<pthread_t, PThread *>::iterator it=_threads.begin(); it!=_threads.end(); it++ ) {
    list.push_back(it->second->bugStack());
  }
  return list;
}

} // namespace GpCoreTools
