/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-06-26
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#ifndef SERIAL_H
#define SERIAL_H

#include <unistd.h>
#include <termios.h>
#include <string>

#include "Stream.h"
#include "DynamicBuffer.h"
#include "GpCoreToolsDLLExport.h"

struct termios;

namespace GpCoreTools {

class GPCORETOOLS_EXPORT Serial : public Stream
{
public:
  Serial(const std::string& devicePath, speed_t speed, tcflag_t flags, DynamicBuffer * buffer=0);
  virtual ~Serial();

  bool connect();

  virtual short eventTypes() {return POLLIN | POLLPRI | POLLRDHUP | POLLHUP;}
  virtual void event(short type);
  virtual void hungUp();

  bool isConnected();

  inline bool write(const char * buf, ssize_t byteCount);
  DynamicBuffer * buffer() const {return _buffer;}

  int lock() {return pthread_mutex_lock(&_mutex);}
  bool tryLock() {return pthread_mutex_trylock(&_mutex)==0;}
  int unlock() {return pthread_mutex_unlock(&_mutex);}
private:
  bool open();
  void init();

  static int outputSpeed(termios * t);

  pthread_mutex_t _mutex;
  DynamicBuffer * _buffer;
  std::string _devicePath;
  speed_t _speed;
  tcflag_t _flags;
};

inline bool Serial::write(const char * buf, ssize_t byteCount)
{
  return ::write(fileDescriptor(), buf, byteCount)==byteCount;
}

} // namespace GpCoreTools

#endif // SERIAL_H
