/***************************************************************************
**
**  This file is part of GPCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2011-06-20
**  Authors:
**    Marc Wathelet (ISTerre, Grenoble, France)
**
***************************************************************************/

#include <assert.h>
#include <errno.h>
#include <string.h>

#include "Trace.h"
#include "Log.h"
#include "Timer.h"
#include "EventLoop.h"
#include "MessageRawHeader.h"
#include "StaticTcpClient.h"

namespace GpCoreTools {

  class StaticTcpClientConnect: public Timer
  {
  public:
    StaticTcpClientConnect(StaticTcpClient * c)
      : Timer() {
      _client=c;
      setInterval(10000);
    }

    virtual bool exec() {
      _client->connect();
      return false; // single shot
    }
  private:
    StaticTcpClient * _client;
  };

  /*!
    \class StaticTcpClient StaticTcpClient.h
    \brief Brief description of class still missing

    Full description of class still missing
  */

  /*!
    Description of constructor still missing
  */
  StaticTcpClient::StaticTcpClient(const Address& host, uint16_t port)
    : TcpClientStream(host)
  {
    TRACE;
    assert(pthread_mutex_init(&_mutex, 0)==0);
    _port=port;
    _connected=false;
    connect();
  }

  void StaticTcpClient::hungUp()
  {
    TRACE;
    pthread_mutex_lock(&_mutex);
    // Sets new socket
    setFileDescriptor(socket(AF_INET, SOCK_STREAM, 0));
    if(fileDescriptor()<=0) {
      Log::write(0, "%s\n", strerror(errno));
    }
    _connected=false;
    pthread_mutex_unlock(&_mutex);
    StaticTcpClientConnect * t=new StaticTcpClientConnect(this);
    t->start(); // Retry in ten seconds
  }

  void StaticTcpClient::connect()
  {
    TRACE;
    pthread_mutex_lock(&_mutex);
    if(!TcpClientStream::connect(_port)) {
      if(EventLoop::instance()) {
        StaticTcpClientConnect * t=new StaticTcpClientConnect(this);
        t->start(); // Retry in ten seconds
      }
    } else {
      _connected=true;
    }
    pthread_mutex_unlock(&_mutex);
  }

  /*!
    Writes buffer inside a packet with head and tail.
  */
  void StaticTcpClient::writePacket(const char * buf, int n)
  {
    TRACE;
    pthread_mutex_lock(&_mutex);
    if(_connected) {
      MessageRawHeader * hdr=new MessageRawHeader(n);
      hdr->littleEndianValues();
      if(writePartial((const char *)hdr, sizeof(MessageRawHeader)) &&
         writePartial(buf, n)) {
        FletcherChecksum chk;
        chk.add(hdr->header(), MessageRawHeader::HeaderSize);
        chk.add(buf, n);
        TcpClientStream::write((const char *)&chk, sizeof(chk));
      }
      delete hdr;
    }
    pthread_mutex_unlock(&_mutex);
  }

  /*!
    Writes buffer inside a packet with head and tail.
    Overload provided for convenience.
  */
  void StaticTcpClient::writePacket(const char * cformat,...)
  {
    TRACE;
    char buf[256];
    va_list argptr;
    va_start(argptr, cformat);
    vsnprintf(buf, 255, cformat, argptr);
    va_end(argptr);
    writePacket(buf, strlen(buf));
  }

  /*!
    Writes plain text.
  */
  void StaticTcpClient::writeText(const char * buf, int n)
  {
    TRACE;
    pthread_mutex_lock(&_mutex);
    if(_connected) {
      TcpClientStream::write(buf, n);
    }
    pthread_mutex_unlock(&_mutex);
  }

  /*!
    Writes plain text.
    Overload provided for convenience
  */
  void StaticTcpClient::writeText(const char * cformat,...)
  {
    TRACE;
    char buf[256];
    va_list argptr;
    va_start(argptr, cformat);
    vsnprintf(buf, 255, cformat, argptr);
    va_end(argptr);
    writeText(buf, strlen(buf));
  }

} // namespace GpCoreTools
