/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-08-19
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <errno.h>
#include <string.h>

#include "TcpClientStream.h"
#include "DynamicBuffer.h"
#include "Log.h"
#include "Trace.h"

namespace GpCoreTools {

/*!
  \class TcpClientStream ClientStream.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
TcpClientStream::TcpClientStream(int fileDescriptor, const Address& peer)
    : Stream()
{
  TRACE;
  setFileDescriptor(fileDescriptor);
  _peer=peer;
  _buffer=0;
}

/*!
  Description of constructor still missing
*/
TcpClientStream::TcpClientStream(const Address& peer)
{
  TRACE;
  setFileDescriptor(socket(AF_INET, SOCK_STREAM, 0));
  if(fileDescriptor()<=0) {
    Log::write(0, "%s\n", strerror(errno));
  }
  _peer=peer;
  _buffer=0;
}

TcpClientStream::~TcpClientStream()
{
  TRACE;
  delete _buffer;
}

bool TcpClientStream::connect(uint16_t port)
{
  TRACE;
  sockaddr_in addr;
  addr.sin_family=AF_INET;
  addr.sin_port=htons(port);
  addr.sin_addr.s_addr=_peer.value();
  Log::write(1, "connecting to %s:%hu (socket %i)\n", _peer.toString().data(), port, fileDescriptor());
  if(::connect(fileDescriptor(), (sockaddr *)&addr, sizeof(sockaddr_in))<0) {
    Log::write(1, "%s\n", strerror(errno));
    return false;
  } else {
    Log::write(1, "connected to %s:%hu (socket %i)\n", inet_ntoa(addr.sin_addr), port, fileDescriptor());
    return true;
  }
}

void TcpClientStream::event(short type)
{
  TRACE;
  if(type & POLLIN) {
    if(_buffer) {
      if(!_buffer->read()) {
        Log::write(1, "error reading data from %s (socket %i)\n",
                   _peer.toString().data(), fileDescriptor());
      }
    }
  }
  if(type & POLLRDHUP) {
    Log::write(1, "connection hung up with %s (socket %i)\n",
               _peer.toString().data(), fileDescriptor());
    close();
    hungUp();
  }
}

void TcpClientStream::setBuffer(DynamicBuffer * buffer)
{
  delete _buffer;
  _buffer=buffer;
}

} // namespace GpCoreTools
