/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-08-19
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <errno.h>
#include <string.h>

#include "TcpServerStream.h"
#include "TcpClientStream.h"
#include "DynamicBuffer.h"
#include "EventLoop.h"
#include "CoreApplication.h"
#include "Log.h"
#include "Trace.h"

namespace GpCoreTools {

/*!
  \class TcpServerStream qtbserverstream.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
TcpServerStream::TcpServerStream()
{
  TRACE;
  setFileDescriptor(socket(AF_INET, SOCK_STREAM, 0));
  if(fileDescriptor()<=0) {
    Log::write(1, "tcp server socket %s\n", strerror(errno));
    CoreApplication::exit(2);
  }
  int optval=1;
  if(setsockopt(fileDescriptor(),SOL_SOCKET,SO_REUSEADDR,&optval,sizeof(optval)) < 0) {
    Log::write(0, "tcp server reuseaddr: [%i] %s\n", errno, strerror(errno));
    CoreApplication::exit(2);
  }
}

bool TcpServerStream::listen(uint16_t port, int maxConnections)
{
  TRACE;
  if(fileDescriptor()<=0) return false;
  sockaddr_in addr;
  addr.sin_family=AF_INET;
  addr.sin_port=htons(port);
  addr.sin_addr.s_addr=htonl(INADDR_ANY);
  if(bind(fileDescriptor(), (sockaddr *)&addr, sizeof(sockaddr_in))<0) {
    Log::write(1, "%s\n", strerror(errno));
    return false;
  }
  if(::listen(fileDescriptor(), maxConnections)<0) {
    Log::write(1, "%s\n", strerror(errno));
    return false;
  } else {
    return true;
  }
}

void TcpServerStream::event(short type)
{
  TRACE;
  if(type & POLLIN) {
    sockaddr_in peerAddr;
    socklen_t peerAddrLen=sizeof(sockaddr_in);
    int peerFd=accept(fileDescriptor(), (sockaddr *)&peerAddr, &peerAddrLen);
    if(peerFd>0) {
      Log::write(2, "connection from %s:%hu (socket %i)\n",inet_ntoa(peerAddr.sin_addr), ntohs(peerAddr.sin_port), peerFd);
      TcpClientStream * s=createClientStream(peerFd, peerAddr.sin_addr.s_addr);
      EventLoop::instance()->addStream(s);
      s->setParent(this);
      _clients.push_back(s);
    } else {
      Log::write(1, "%s\n", strerror(errno));
    }
  }
}

void TcpServerStream::childDeleted(Stream * client)
{
  TRACE;
  std::list<TcpClientStream *>::iterator it;
  for(it=_clients.begin();it!=_clients.end();it++) {
    if(*it==client) {
      break;
    }
  }
  if(it!=_clients.end()) {
    _clients.erase(it);
  }
}

/*!
  Send the same packet to all connected clients
*/
void TcpServerStream::send(const char * bytes, int byteCount)
{
  TRACE;
  std::list<TcpClientStream *>::iterator it;
  for(it=_clients.begin();it!=_clients.end();it++) {
    (*it)->buffer()->send(bytes, byteCount);
  }
}

/*!
  Send the same packet to all connected clients
*/
void TcpServerStream::sendPartial(const char * bytes, int byteCount)
{
  TRACE;
  std::list<TcpClientStream *>::iterator it;
  for(it=_clients.begin();it!=_clients.end();it++) {
    (*it)->buffer()->sendPartial(bytes, byteCount);
  }
}

} // namespace GpCoreTools
