/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-08-20
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <stdio.h>

#include "Timer.h"
#include "EventLoop.h"
#include "Trace.h"

namespace GpCoreTools {

/*!
  \class Timer qtbtimer.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
Timer::Timer()
{
  TRACE;
  _interval=1000;
  _queued=0;
}

Timer::~Timer()
{
  TRACE;
  stop();
}

/*!
  \fn Timer::setInterval(int i)
  Sets delay to \a i ms.
*/

/*!
  \fn Timer::exec()
  Performs action after timeout. If it returns true the timer is not single shot,
  i.e. deleted right after exec().
*/

void Timer::start()
{
  TRACE;
  stop();
  _queued=EventLoop::instance()->addTimer(this);
}

void Timer::stop()
{
  TRACE;
  if(_queued) {
    _queued->detach();
    _queued=0;
  }
}

void Timer::setTimeout()
{
  TRACE;
  clock_gettime(CLOCK_REALTIME, &_timeout);
  int sec=_interval/1000;
  long nsec=(_interval-sec*1000)*1000000;
  _timeout.tv_nsec+=nsec;
  _timeout.tv_sec+=sec;
  if(_timeout.tv_nsec>1000000000) {
    _timeout.tv_sec++;
    _timeout.tv_nsec-=1000000000;
  }
}

int Timer::timeout() const
{
  TRACE;
  timespec t;
  clock_gettime(CLOCK_REALTIME, &t);
  int d=(_timeout.tv_sec-t.tv_sec)*1000+(_timeout.tv_nsec-t.tv_nsec)/1000000;
  return d<0 ? 0 : d;
}

bool Timer::after(Timer * o) const
{
  TRACE;
  if(_timeout.tv_sec>o->_timeout.tv_sec) {
    return true;
  } else if(_timeout.tv_sec<o->_timeout.tv_sec) {
    return false;
  } else {
    if(_timeout.tv_nsec>o->_timeout.tv_nsec) {
      return true;
    } else {
      return false;
    }
  }
}

void Timer::printDebug()
{
  TRACE;
  printf("Timer interval=%i, abs_timeout_sec=%li, abs_timeout_nsec=%li, rel_timeout=%i, running=%s\n",
         _interval, _timeout.tv_sec, _timeout.tv_nsec, timeout(), _queued ? "yes" : "no");
}

} // namespace GpCoreTools
