/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2007-12-26
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include "Trace.h"
#include "PThread.h"
#include "Log.h"
#include "CoreApplication.h"

namespace GpCoreTools {

#ifdef TRACE_ENABLED

/*!
  \class Trace Trace.h
  \brief Brief description of class still missing

  To get a debug trace of all functions, you can define in an implementation files (.cpp):
  \pre
    #ifdef TRACE
    #  undef TRACE
    #endif
    #define TRACE printf("%s\n",__PRETTY_FUNCTION__);
  \pre
*/

std::vector<const TraceStamp *> Trace::_mainStack;

/*!
  Description of constructor still missing
*/
Trace::Trace(const TraceStamp * s)
{
  PThread * t=PThread::currentThread();
  if(t) {
    _myStack=t->stack();
    _myStack->push_back(s);
  } else {
    _myStack=&_mainStack;
    _myStack->push_back(s);
  }
  //printf("[TRACE] %s\n",s->toString().toLatin1().data());
  //fflush(stdout);
}

/*!
  Description of destructor still missing
*/
Trace::~Trace()
{
  if(_myStack) {
    _myStack->pop_back();
  }
}

void Trace::backTrace(const std::vector<const TraceStamp *> * stack)
{
  for(std::vector<const TraceStamp *>::const_iterator it=stack->begin();it!=stack->end();it++) {
    Log::write(0, (*it)->toString().data());
  }
}

#endif

} // namespace GpCoreTools
