/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2010-04-08
**  Authors:
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <errno.h>
#include <string.h>

#include "UdpServerStream.h"
#include "DynamicBuffer.h"
#include "EventLoop.h"
#include "CoreApplication.h"
#include "Log.h"
#include "Trace.h"

namespace GpCoreTools {

/*!
  \class UpdServerStream UpdServerStream.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
UdpServerStream::UdpServerStream()
    : Stream()
{
  TRACE;
  setFileDescriptor(socket(AF_INET, SOCK_DGRAM, 0));
  if(fileDescriptor()<=0) {
    Log::write(1, "udp server socket %s\n", strerror(errno));
    CoreApplication::exit(2);
  }
  int optval=1;
  if(setsockopt(fileDescriptor(),SOL_SOCKET,SO_REUSEADDR,&optval,sizeof(optval)) < 0) {
    Log::write(0, "udp server reuseaddr: [%i] %s\n", errno, strerror(errno));
    CoreApplication::exit(2);
  }
}

bool UdpServerStream::listen(uint16_t port)
{
  TRACE;
  if(fileDescriptor()<=0) return false;
  sockaddr_in addr;
  addr.sin_family=AF_INET;
  addr.sin_port=htons(port);
  addr.sin_addr.s_addr=htonl(INADDR_ANY);
  if(bind(fileDescriptor(), (sockaddr *)&addr, sizeof(sockaddr_in))<0) {
    Log::write(1, "udp server listen: %s\n", strerror(errno));
    return false;
  } else {
    return true;
  }
}

bool UdpServerStream::send(uint16_t port, const char * bytes, int byteCount)
{
  TRACE;
  int fd=socket(AF_INET,SOCK_DGRAM,0);
  if(fd<0) {
    Log::write(0, "udp send socket error: [%i] %s\n", errno, strerror(errno));
    return false;
  }
  int optval=1;
  if(setsockopt(fd, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval))<0) {
    Log::write(0, "udb broadcast: [%i] %s\n", errno, strerror(errno));
    return false;
  }
  struct sockaddr_in addr;
  addr.sin_family=AF_INET;
  addr.sin_addr.s_addr=Address::me().subnet().value();
  addr.sin_port=htons(port);
  if(sendto(fd, bytes, byteCount, 0,(struct sockaddr *) &addr, sizeof(addr))<0) {
    Log::write(0, "udp sendto error: %s\n", strerror(errno));
    ::close(fd);
    return false;
  } else {
    ::close(fd);
    return true;
  }
}

} // namespace GpCoreTools
