/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2010-02-02
**  Authors:
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <errno.h>
#include <string.h>
#include <assert.h>

#include "UnixClientStream.h"
#include "Log.h"
#include "Trace.h"

namespace GpCoreTools {

/*!
  \class UnixClientStream UnixClientStream.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
UnixClientStream::UnixClientStream(int fileDescriptor, const char * peer)
    : Stream()
{
  TRACE;
  setFileDescriptor(fileDescriptor);
  setAddress(_peer, peer);
  _buffer=0;
}

/*!
  Description of constructor still missing
*/
UnixClientStream::UnixClientStream(const char * peer)
    : Stream()
{
  TRACE;
  setFileDescriptor(socket(AF_UNIX, SOCK_STREAM, 0));
  if(fileDescriptor()<=0) {
    Log::write(0, "%s\n", strerror(errno));
  }
  setAddress(_peer, peer);
  _buffer=0;
}

/*!
  Description of destructor still missing
*/
UnixClientStream::~UnixClientStream()
{
  TRACE;
  delete _buffer;
}

void UnixClientStream::setAddress(sockaddr_un& addr, const char * peer)
{
  addr.sun_family=AF_UNIX;
  assert(strlen(peer)<=108-1);
  memset(addr.sun_path, 108*sizeof(char), 0);
  strcpy(addr.sun_path, peer);
}

bool UnixClientStream::connect()
{
  TRACE;
  Log::write(1, "connecting to %s\n", _peer.sun_path);
  if(::connect(fileDescriptor(), (sockaddr *)&_peer, sizeof(sockaddr_un))<0) {
    Log::write(1, "%s\n", strerror(errno));
    return false;
  } else {
    Log::write(1, "connected to %s\n", _peer.sun_path);
    return true;
  }
}

void UnixClientStream::event(short type)
{
  TRACE;
  if(type & POLLIN) {
    if(!_buffer->read()) {
      Log::write(1, "error reading data (socket %i)\n", fileDescriptor());
    }
  }
  if(type & POLLRDHUP) {
    Log::write(1, "connection hung up with %s (%i)\n",
               _peer.sun_path, fileDescriptor());
    close();
    hungUp();
  }
}

} // namespace GpCoreTools
