/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2010-02-02
**  Authors:
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <errno.h>
#include <string.h>
#include <sys/un.h>
#include <sys/socket.h>

#include "UnixServerStream.h"
#include "UnixClientStream.h"
#include "Log.h"
#include "EventLoop.h"
#include "CoreApplication.h"
#include "Trace.h"

namespace GpCoreTools {

/*!
  \class UnixServerStream UnixServerStream.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

/*!
  Description of constructor still missing
*/
UnixServerStream::UnixServerStream()
    : Stream()
{
  TRACE;
  setFileDescriptor(socket(AF_UNIX, SOCK_STREAM, 0));
  if(fileDescriptor()<=0) {
    Log::write(1, "server socket %s\n", strerror(errno));
    CoreApplication::exit(2);
  }
  int optval=1;
  if(setsockopt(fileDescriptor(),SOL_SOCKET,SO_REUSEADDR,&optval,sizeof(optval)) < 0) {
    Log::write(0, "peer tracker reuseaddr: [%i] %s\n", errno, strerror(errno));
    CoreApplication::exit(2);
  }
}


bool UnixServerStream::listen(const char * name, int maxConnections)
{
  TRACE;
  if(fileDescriptor()<=0) return false;
  sockaddr_un addr;
  UnixClientStream::setAddress(addr, name);
  if(bind(fileDescriptor(), (sockaddr *)&addr, sizeof(sockaddr_un))<0) {
    Log::write(1, "%s\n", strerror(errno));
    return false;
  }
  if(::listen(fileDescriptor(), maxConnections)<0) {
    Log::write(1, "%s\n", strerror(errno));
    return false;
  } else {
    return true;
  }
}

void UnixServerStream::event(short type)
{
  TRACE;
  if(type & POLLIN) {
    sockaddr_un peerAddr;
    socklen_t peerAddrLen=sizeof(sockaddr_un);
    int peerFd=accept(fileDescriptor(), (sockaddr *)&peerAddr, &peerAddrLen);
    if(peerFd>0) {
      if(peerAddrLen==sizeof(sa_family_t)) { // unamed socket
        Log::write(2, "connection from unamed peer\n");
      } else {
        Log::write(2, "connection from %s\n", peerAddr.sun_path);
      }
      UnixClientStream * s=createClientStream(peerFd, peerAddr.sun_path);
      EventLoop::instance()->addStream(s);
      s->setParent(this);
      _clients.push_back(s);
    } else {
      Log::write(1, "%s\n", strerror(errno));
    }
  }
}

void UnixServerStream::childDeleted(Stream * client)
{
  TRACE;
  std::list<UnixClientStream *>::iterator it;
  for(it=_clients.begin();it!=_clients.end();it++) {
    if(*it==client) {
      break;
    }
  }
  if(it!=_clients.end()) {
    _clients.erase(it);
  }
}

/*!
  Send the same packet to all connected clients
*/
void UnixServerStream::send(const char * bytes, int byteCount)
{
  TRACE;
  std::list<UnixClientStream *>::iterator it;
  for(it=_clients.begin();it!=_clients.end();it++) {
    (*it)->buffer()->send(bytes, byteCount);
  }
}

} // namespace GpCoreTools
