/***************************************************************************
**
**  This file is part of GpCoreTools.
**
**  This file may be distributed and/or modified under the terms of the
**  GNU General Public License version 2 or 3 as published by the Free
**  Software Foundation and appearing in the file LICENSE.GPL included
**  in the packaging of this file.
**
**  This file is distributed in the hope that it will be useful, but WITHOUT
**  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
**  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
**  more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**  See http://www.geopsy.org for more information.
**
**  Created: 2009-12-02
**  Authors:
**    Marc Wathelet
**    Marc Wathelet (LGIT, Grenoble, France)
**
***************************************************************************/

#include <stdio.h>

#include "Variant.h"

namespace GpCoreTools {

/*!
  \class Variant Variant.h
  \brief Brief description of class still missing

  Full description of class still missing
*/

inline void Variant::clearInternal()
{
  switch(_type) {
  case String:
    delete reinterpret_cast<std::string *>(_data);
    break;
  case Int:
    delete reinterpret_cast<int *>(_data);
    break;
  case Long:
    delete reinterpret_cast<long *>(_data);
    break;
  case Double:
    delete reinterpret_cast<int *>(_data);
    break;
  case Unknown:
    break;
  }
}

inline void Variant::assignInternal(const Variant& o)
{
  _type=o._type;
  switch(_type) {
  case String:
    _data=new std::string(*reinterpret_cast<std::string *>(o._data));
    break;
  case Int:
    _data=new int;
    *reinterpret_cast<int *>(_data)=*reinterpret_cast<int *>(o._data);
    break;
  case Long:
    _data=new long;
    *reinterpret_cast<long *>(_data)=*reinterpret_cast<long *>(o._data);
    break;
  case Double:
    _data=new double;
    *reinterpret_cast<double *>(_data)=*reinterpret_cast<double *>(o._data);
    break;
  case Unknown:
    _data=0;
    break;
  }
}

Variant::~Variant()
{
  clearInternal();
}

Variant::Variant(const Variant& o)
{
  assignInternal(o);
}

void Variant::operator=(const Variant& o)
{
  clearInternal();
  assignInternal(o);
}

std::string Variant::toString() const
{
  switch(_type) {
  case String:
    return *reinterpret_cast<std::string *>(_data);
  case Int: {
      char str[16];
      sprintf(str, "%i", *reinterpret_cast<int *>(_data));
      return str;
    }
  case Long: {
      char str[32];
      sprintf(str, "%li", *reinterpret_cast<long *>(_data));
      return str;
    }
  case Double:{
      char str[512]; // Maximum double around 1.8e+308
      sprintf(str, "%.20lf", *reinterpret_cast<double *>(_data));
      return str;
    }
  case Unknown:
    break;
  }
  return "";
}

} // namespace GpCoreTools
